from ..base.generator_data_storage_base import GeneratorDataStorageBase


class GeneratorDataStorage(GeneratorDataStorageBase):

    def __init__(self):
        GeneratorDataStorageBase.__init__(self)

    def get_shared_method_body(self):
        return '''        if not DataStorage.__instance:
    DataStorage.__instance = DataStorage()
return DataStorage.__instance'''

    def get_pattern_getter(self):
        return '''        if not self._loaded and name not in self.{map}:
            from .{type} import {type}
            self.{map}[name] = {type}()
            self.{map}[name].name = name
        return self.{map}[name]'''

    def get_initialize_function_json_body(self):
        return '''        js = json.loads(content)
        self.deserialize_json(js)
        self._loaded = True'''

    def get_initialize_function_xml_body(self):
        return '''        root = ET.fromstring(content)
        self.deserialize_xml(root)
        self._loaded = True
        '''



from .writer import Writer


class GeneratorFactory(object):

    def __init__(self):
        pass

    @staticmethod
    def generate(model):
        writer = Writer('')
        writer.model = model
        content = writer.prepare_file(FACTORY)
        model.add_file(None, 'Factory.js', content)


FACTORY = '''
class Factory
{
}

Factory.build = function (type)
{
    return eval("new " + type + "()");
};
Factory.create_command_from_json = function (payload)
{
    let json = JSON.parse(payload);
    for(let key in json)
    {
        let command = Factory.build(key);
        if(command)
        {
            command.deserialize_json(json[key]);
        }
        return command;
    }
};

Factory.serialize_command_to_json = function (command)
{
    let json = {};
    json[command.get_type()] = {};
    command.serialize_json(json[command.get_type()]);
    return JSON.stringify(json);
};

Factory.clone_object = function (obj)
{
    let payload = Factory.serialize_command_to_json(obj);
    return Factory.create_command_from_json(payload);
};
'''



# from setuptools import setup, find_packages
# import mlc_tools.version

# setup(
#     name='mlc-tools',
#     version=mlc_tools.version,
#     packages=find_packages(),
#     long_description="mlc-tools",
#     install_requires=[
#         'enum'
#     ]
#     # entry_points={
#     #     'console_scripts':
#     #         ['mlc-tools = mlc_tools.main:console']
#     # },
#     # test_suite='tests'
# )
import setuptools
import mlc_tools.version

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="mlc-tools",
    version=mlc_tools.version,
    author="Vladimir Tolmachev",
    author_email="tolm_vl@hotmail.com",
    description="A tool to generate and translate C++ code to other languages from one code-base.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mlc-tools/mlc-tools",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)